import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.Scanner;

public class Main
{
	static class Cell {
		int color;
		int price;
	}
	
	static class Offset {
		int or;
		int oc;
		long estimate;

		public Offset(int or, int oc, long estimate) {
			this.or = or;
			this.oc = oc;
			this.estimate = estimate;
		}
	}
	
	static class Solution {
		long score = 0;
		StringBuilder output = new StringBuilder();
	}
	
	static Solution smallBoardSolution(int n, Cell[][] board) {
		Solution res = new Solution();
		Cell[][] bc = new Cell[n][n];
		for (int row = 0; row < n; ++row) {
			for (int col = 0; col < n; ++col) {
				bc[row][col] = board[row][col];
			}
		}
		board = bc;
		int[] totalPriceByColor = new int[10];
		for (int row = 0; row < n; ++row) {
			for (int col = 0; col < n; ++col) {
				Cell cell = board[row][col];
				totalPriceByColor[cell.color] += cell.price;
			}
		}
		long theoretical = 0;
		for (int i = 0; i < totalPriceByColor.length; ++i) {
			theoretical += totalPriceByColor[i] * (long) totalPriceByColor[i];
		}
		int moveCount = 0;
		while (true) {
			boolean[][] visited = new boolean[n][n];
			int maxSize = 0;
			int maxColor = -1;
			int maxComplete = -1;
			int maxRow = -1;
			int maxCol = -1;			
			for (int col = 0; col < n; ++col) {
				for (int row = 0; row < n; ++row) {
					if (board[row][col] != null && !visited[row][col]) {
						int size = dfs(row, col, n, board, visited);
						int color = board[row][col].color;
						int complete = size == totalPriceByColor[color] ? 1 : 0;
						if ((maxComplete < 0) || (complete > maxComplete) || (complete == maxComplete && totalPriceByColor[color] < totalPriceByColor[maxColor])) {
							maxSize = size;
							maxColor = color;
							maxComplete = complete;
							maxRow = row;
							maxCol = col;
						}
					}
				}
			}
			if (maxSize == 0) break;
			visited = new boolean[n][n];
			dfs(maxRow, maxCol, n, board, visited);
			res.score += maxSize * (long) maxSize;
			++moveCount;
			res.output.append(maxRow + " " + maxCol + "\n");
			Cell[][] newBoard = new Cell[n][n];
			int nextOutputCol = 0;
			for (int col = 0; col < n; ++col) {
				int nextOutputRow = n - 1;
				for (int row = n - 1; row >= 0; --row) if (board[row][col] != null && !visited[row][col]) {
					newBoard[nextOutputRow][nextOutputCol] = board[row][col];
					--nextOutputRow;
				}
				if (nextOutputRow < n - 1)
					++nextOutputCol;
			}
			board = newBoard;
		}
		res.output = new StringBuilder(moveCount + "\n" + res.output.toString());
		return res;
	}
	
	private static int dfs(int row, int col, int n, Cell[][] board, boolean[][] visited) {
		int res = board[row][col].price;
		visited[row][col] = true;
		for (int dr = -1; dr <= 1; ++dr) {
			for (int dc = -1; dc <= 1; ++dc) if (Math.abs(dr) + Math.abs(dc) == 1) {
				int nr = row + dr;
				int nc = col + dc;
				if (nr >= 0 && nr < n && nc >= 0 && nc < n && board[nr][nc] != null
						&& board[nr][nc].color == board[row][col].color && !visited[nr][nc]) {
					res += dfs(nr, nc, n, board, visited);
				}
			}
		}
		return res;
	}
	
	static interface Evaluator {
		long evaluate(int color, long count);
	}
	
	public static void main(String[] args) throws IOException
	{
		long start = System.currentTimeMillis();
		Scanner scanner = new Scanner(new File("colgame.in"));
		PrintWriter pw = new PrintWriter("colgame.out");
		int n = scanner.nextInt();
		n *= 3;
		Cell[][] board = new Cell[n][n];
		for (int row = 0; row < n; ++row) {
			for (int col = 0; col < n; ++col) {
				board[row][col] = new Cell();
				board[row][col].color = scanner.nextInt() - 1;
			}
		}
		for (int row = 0; row < n; ++row) {
			for (int col = 0; col < n; ++col) {
				board[row][col].price = scanner.nextInt();
			}
		}
		n /= 3;
		
		int[] totalPriceByColor = new int[10];
		for (int row = 0; row < 3 * n; ++row) {
			for (int col = 0; col < 3 * n; ++col) {
				Cell cell = board[row][col];
				totalPriceByColor[cell.color] += cell.price;
			}
		}
		
		double maxScore = 0;
		int maxColor = 0;
		for (int i = 0; i < 10; ++i) {
			double t = totalPriceByColor[i];
			maxScore += t * t * t * t;
			System.err.print(totalPriceByColor[i] + " ");
			if (totalPriceByColor[i] > totalPriceByColor[maxColor]) {
				maxColor = i;
			}
		}
		System.err.println();
		double divBy = Math.sqrt(Math.max(1, maxScore/ 7e15));
		final long divByLong = Math.round(divBy);
		final double maxScoreCopy = maxScore;
		double checkPr = Math.min(50, n) / (double) n;
		Random random = new Random(541354315413L); 
		
		/*Evaluator ev = new Evaluator() {
			@Override
			public long evaluate(int color, long count) {
				long z = count;
				z *= z;
				z /= divByLong;
				z *= z;
				if (color < 10) z *= 1000; else if (color < 40 || (color >= 60 && color < 70)) z *= 1; else z *= 0;
				return z;
			}
		};*/
		
		int[] bestWho = null;
		long bestScore = -1;
		int[][] interesting = new int[3 * n][];
		for (int row = 0; row < 3 * n; ++row) {
			int cnt = 0;
			for (int col = 0; col < 3 * n; ++col) if (board[row][col].color == maxColor) ++cnt;
			interesting[row] = new int[cnt];
			cnt = 0;
			for (int col = 0; col < 3 * n; ++col) if (board[row][col].color == maxColor) interesting[row][cnt++] = col;
		}
		int attempts = Math.min(1000, 50 * 50 * 50 / n / n / n);
		for (int att = 0; att < attempts; ++att) {
			int[] who = new int[n];
			for (int i = 0; i < n; ++i) {
				who[i] = random.nextInt(3);
			}
			long score = evalBestRotation(n, board, who, interesting);
			int toCheck = 0;
			while (true) {
				--toCheck;
				if (toCheck < 0) {
					if (System.currentTimeMillis() >= start + 2000 * (att + 1) / attempts) break;
					toCheck = attempts;
				}
				int[] newWho = who.clone();
				int ni = random.nextInt(n);
				int nv = (who[ni] + random.nextInt(2) + 1) % 3;
				newWho[ni] = nv;
				if (random.nextInt(5) == 0) {
					int ni2;
					do {
						ni2 = random.nextInt(n);
					} while (ni2 == ni);
					int nv2 = (who[ni2] + random.nextInt(2) + 1) % 3;
					newWho[ni2] = nv2;
				}
				long nscore = evalBestRotation(n, board, newWho, interesting);
				if (nscore > score) {
					score = nscore;
					who = newWho;
					//System.err.println(score);
				}			
			}
			Cell[][] rotBoard = applyBestRotation(n, board, who, interesting, null);
			score = smallBoardSolution(n, rotBoard).score;
			if (score > bestScore) {
				bestScore = score;
				bestWho = who;
			}
		}
		board = applyBestRotation(n, board, bestWho, interesting, pw);
		
		/*for (int rotation = 0; rotation < 3 * n; ++rotation) {
			int[] counts = new int[90];
			for (int row = 0; row < 3 * n; ++row) {
				for (int col = 0; col < 3 * n; ++col) {
					Cell cell = board[row][col];
					counts[row / n * 30 + col / n * 10 + cell.color] += cell.price; 
				}
			}
			long score = 0;
			for (int i = 0; i < counts.length; ++i)
				score += ev.evaluate(i, counts[i]);
			long bestScore = score;
			int bestKind = 0;
			int bestItem = 0;
			int bestBy = 0;
			{
				for (int row = 0; row < 3 * n; ++row) if (random.nextDouble() < checkPr) {
					for (int by = 1; by <= 3 * n; ++by) {
						for (int block = 0; block < 3; ++block) {
							int blockOffset = row / n * 30 + block * 10;
							int loose = block * n + n - by;
							while (loose >= 3 * n) loose -= 3 * n;
							while (loose < 0) loose += 3 * n;
							int win = block * n - by;
							while (win >= 3 * n) win -= 3 * n;
							while (win < 0) win += 3 * n;
							Cell looseCell = board[row][loose];
							Cell winCell = board[row][win];
							score -= ev.evaluate(blockOffset + looseCell.color, counts[blockOffset + looseCell.color]);
							counts[blockOffset + looseCell.color] -= looseCell.price;
							score += ev.evaluate(blockOffset + looseCell.color, counts[blockOffset + looseCell.color]);
							score -= ev.evaluate(blockOffset + winCell.color, counts[blockOffset + winCell.color]);
							counts[blockOffset + winCell.color] += winCell.price;
							score += ev.evaluate(blockOffset + winCell.color, counts[blockOffset + winCell.color]);
						}
						if (score < 0) throw new RuntimeException();
						if (score > bestScore) {
							bestScore = score;
							bestKind = 0;
							bestItem = row;
							bestBy = by;
						}
					}
				}
			}
			{
				for (int col = 0; col < 3 * n; ++col) if (random.nextDouble() < checkPr) {
					for (int by = 1; by <= 3 * n; ++by) {
						for (int block = 0; block < 3; ++block) {
							int blockOffset = col / n * 10 + block * 30;
							int loose = block * n + n - by;
							while (loose >= 3 * n) loose -= 3 * n;
							while (loose < 0) loose += 3 * n;
							int win = block * n - by;
							while (win >= 3 * n) win -= 3 * n;
							while (win < 0) win += 3 * n;
							Cell looseCell = board[loose][col];
							Cell winCell = board[win][col];
							score -= ev.evaluate(blockOffset + looseCell.color, counts[blockOffset + looseCell.color]);
							counts[blockOffset + looseCell.color] -= looseCell.price;
							score += ev.evaluate(blockOffset + looseCell.color, counts[blockOffset + looseCell.color]);
							score -= ev.evaluate(blockOffset + winCell.color, counts[blockOffset + winCell.color]);
							counts[blockOffset + winCell.color] += winCell.price;
							score += ev.evaluate(blockOffset + winCell.color, counts[blockOffset + winCell.color]);
						}
						if (score < 0) throw new RuntimeException();
						if (score > bestScore) {
							bestScore = score;
							bestKind = 1;
							bestItem = col;
							bestBy = by;
						}
					}
				}
			}
			pw.println(bestKind + " " + bestItem + " " + bestBy);
			Cell[][] newBoard = new Cell[3 * n][3 * n];
			for (int row = 0; row < 3 * n; ++row) {
				for (int col = 0; col < 3 * n; ++col) {
					Cell cell = board[row][col];
					if (bestKind == 0 && (row == bestItem)) {
						int newCol = (col + bestBy) % (3 * n);
						newBoard[row][newCol] = cell;
					} else if (bestKind == 1 && (col == bestItem)) {
						int newRow = (row + bestBy) % (3 * n);
						newBoard[newRow][col] = cell;
					} else {
						newBoard[row][col] = cell;
					}
				}
			}
			board = newBoard;
		}*/
		
		Solution best = null;
		int bor = -1;
		int boc = -1;
		List<Offset> offsets = new ArrayList<Offset>(); 
		for (int or = 0; or <= 2 * n; ++or) {
			for (int oc = 0; oc <= 2 * n; ++oc) {
				int[] smallTotal = new int[11];
				for (int row = 0; row < n; ++row) {
					for (int col = 0; col < n; ++col) {
						Cell cell = board[or + row][oc + col];
						smallTotal[cell.color] += cell.price;
					}
				}
				long score = 0;
				/*for (int i = 0; i < smallTotal.length; ++i) {
					score += ev.evaluate(i, smallTotal[i]);
				}*/
				offsets.add(new Offset(or, oc, score));
			}
		}
		Collections.sort(offsets, new Comparator<Offset>(){
			@Override
			public int compare(Offset arg0, Offset arg1) {
				int z = Long.compare(arg1.estimate, arg0.estimate);
				if (z == 0) z = arg0.or - arg1.or;
				if (z == 0) z = arg0.oc - arg1.oc;
				return z;
			}});
				
		int offsetPtr = 0;
		System.err.println("Time elapsed already: " + (System.currentTimeMillis() - start));
		while (offsetPtr < offsets.size() && System.currentTimeMillis() < start + 3000) {
			int or = offsets.get(offsetPtr).or;
			int oc = offsets.get(offsetPtr).oc;
			++offsetPtr;
			Cell[][] smallBoard = new Cell[n][n];
			for (int row = 0; row < n; ++row) {
				for (int col = 0; col < n; ++col) {
					smallBoard[row][col] = board[or + row][oc + col];
				}
			}
			Solution sol = smallBoardSolution(n, smallBoard);
			if (best == null || sol.score > best.score) {
				best = sol;
				bor = or;
				boc = oc;
			}
			//break;
		}
		
		pw.println(bor + " " + boc);
		pw.println(best.output.toString());
		pw.close();
		scanner.close();
		
		System.err.println("Expected score: " + best.score);
	}

	private static long evalBestRotation(int n, Cell[][] board, int[] who,
			int[][] interesting) {
		int availableMoves = 3 * n;
		for (int i = 0; i < n; ++i) if (who[i] > 0) --availableMoves;
		int[][] scores = new int[3 * n][3 * n];
		for (int i = 0; i < n; ++i) {
			for (int row = who[i] * n; row < (who[i] + 1) * n; ++row) {
				for (int col : interesting[row]) {
					int rot = i - col;
					if (rot < 0) rot += 3 * n;
					scores[row][rot] += board[row][col].price;
				}
			}
		}
		int[] best = new int[availableMoves + 1];
		for (int row = 0; row < 3 * n; ++row) {
			int bestZero = scores[row][0];
			int bestOne = 0;
			for (int i = 1; i < 3 * n; ++i)
				if (scores[row][i] > bestOne) {
					bestOne = scores[row][i];
				}
			for (int i = availableMoves; i >= 0; --i) {
				if (i + 1 <= availableMoves) {
					best[i + 1] = Math.max(best[i + 1], best[i] + bestOne);
				}
				best[i] += bestZero;
			}
		}
		return best[availableMoves];
	} 

	private static Cell[][] applyBestRotation(int n, Cell[][] board, int[] who,
			int[][] interesting, PrintWriter pw) {
		int availableMoves = 3 * n;
		for (int i = 0; i < n; ++i) if (who[i] > 0) --availableMoves;
		int[][] scores = new int[3 * n][3 * n];
		for (int i = 0; i < n; ++i) {
			for (int row = who[i] * n; row < (who[i] + 1) * n; ++row) {
				for (int col : interesting[row]) {
					int rot = i - col;
					if (rot < 0) rot += 3 * n;
					scores[row][rot] += board[row][col].price;
				}
			}
		}
		int[][] best = new int[3 * n + 1][availableMoves + 1];
		int[][] via = new int[3 * n + 1][availableMoves + 1];
		int[] bestOneRot = new int[3 * n];
		for (int row = 0; row < 3 * n; ++row) {
			int bestZero = scores[row][0];
			int bestOne = -1;
			bestOneRot[row] = -1;
			for (int i = 1; i < 3 * n; ++i)
				if (scores[row][i] > bestOne) {
					bestOne = scores[row][i];
					bestOneRot[row] = i;
				}
			for (int i = availableMoves; i >= 0; --i) {
				if (i + 1 <= availableMoves) {
					if (best[row + 1][i + 1] < best[row][i] + bestOne) {
						best[row + 1][i + 1] = best[row][i] + bestOne;
						via[row + 1][i + 1] = 1;
					}
				}
				best[row + 1][i] = best[row][i] + bestZero;
			}
		}
		int wroteMoves = 0;
		int av = availableMoves;
		Cell[][] newBoard = new Cell[3 * n][3 * n];
		for (int row = 3 * n - 1; row >= 0; --row) {
			int vi = via[row + 1][av];
			if (vi == 1) {
				if (pw != null) pw.println("0 " + row + " " + bestOneRot[row]);
				for (int i = 0; i < 3 * n; ++i) {
					newBoard[row][(i + bestOneRot[row]) % (3 * n)] = board[row][i];
				}
				++wroteMoves;
				--av;
			} else {
				for (int i = 0; i < 3 * n; ++i) {
					newBoard[row][i] = board[row][i];
				}
			}
		}
		for (int i = 0; i < n; ++i) if (who[i] > 0) {
			if (pw != null) pw.println("1 " + i + " " + (3 - who[i]) * n);
			++wroteMoves;
			Cell[] tmp = new Cell[3 * n];
			for (int j = 0; j < 3 * n; ++j) tmp[(j + ((3 - who[i]) * n)) % (3 * n)] = newBoard[j][i];
			for (int j = 0; j < 3 * n; ++j) newBoard[j][i] = tmp[j];
		}
		if (pw != null) System.err.println("Wrote moves: " + wroteMoves);
		if (wroteMoves > 3 * n) throw new RuntimeException();
		while (wroteMoves < 3 * n) {
			if (pw != null) pw.println("0 0 0");
			++wroteMoves;
		}
		return newBoard;
	} 
}